@@ -71,7 +71,7 @@ DEFCHECKER(check_gdbm_errno, prefix=GDBM, NO-ERROR MALLOC-ERROR              \
            REORGANIZE-FAILED CANNOT-REPLACE ILLEGAL-DATA OPT-ALREADY-SET     \
            OPT-ILLEGAL)
 nonreturning_function(static, error_gdbm, (char *fatal_message)) {
-  end_system_call(); /* in case we are called from _gdbm_fatal() */
+  end_blocking_system_call(); /* in case we are called from _gdbm_fatal() */
   pushSTACK(`GDBM::GDBM-ERROR`);
   pushSTACK(`:MESSAGE`);
   if (fatal_message) {
@@ -122,7 +122,7 @@ static GDBM_FILE check_gdbm (gcv_object_t *gdbm, gdbm_data_t *key,
   }
 }
 
-#define SYSCALL(statement) begin_system_call(); statement; end_system_call()
+#define SYSCALL(statement) begin_blocking_system_call(); statement; end_blocking_system_call()
 
 static object open_gdbm (object path, int bsize, int rw, int mode) {
   GDBM_FILE gdbm;
@@ -149,10 +149,11 @@ DEFUN(GDBM::GDBM-OPEN, name &key BLOCKSIZE READ-WRITE OPTION MODE   \
   int rw = rw_opt1 | rw_opt2;
   int bsize = check_uint_defaulted(popSTACK(), 512);
   if (typep_classname(STACK_0,`GDBM::GDBM`)) { /* reuse */
-    if (!check_gdbm(&STACK_0,&default_key_type,&default_value_type,false))
-      TheStructure(STACK_0)->recdata[GDBM_SLOT_FILE] = /* reopen */
-        open_gdbm(TheStructure(STACK_0)->recdata[GDBM_SLOT_PATH],
-                  bsize, rw, mode);
+    if (!check_gdbm(&STACK_0,&default_key_type,&default_value_type,false)) {
+      value1 = open_gdbm(TheStructure(STACK_0)->recdata[GDBM_SLOT_PATH],
+                         bsize, rw, mode); /* reopen */
+      TheStructure(STACK_0)->recdata[GDBM_SLOT_FILE] = value1;
+    }
     TheStructure(STACK_0)->recdata[GDBM_SLOT_KEY]=fixnum(default_key_type);
     TheStructure(STACK_0)->recdata[GDBM_SLOT_VAL]=fixnum(default_value_type);
     VALUES1(popSTACK());        /* return the argument */
@@ -200,7 +201,7 @@ DEFUN(GDBM:GDBM-FILE-SIZE, dbf)
 {
   GDBM_FILE dbf = check_gdbm(&STACK_0,NULL,NULL,true);
   off_t ret;
-  SYSCALL(ret = handle_length(NIL,gdbm_fdesc(dbf)));
+  SYSCALL(ret = handle_length(NULL,gdbm_fdesc(dbf)));
   VALUES1(off_to_I(ret)); skipSTACK(1);
 }
 #endif  /* HAVE_GDBM_FDESC */
