@@ -23,7 +23,7 @@
 ;; SOFTWARE.
 
 ;;; current license:
-;; Copyright (C) 2006 Sam Steingold (sds@gnu.org)
+;; Copyright (C) 2006-2009 Sam Steingold (sds@gnu.org)
 ;;
 ;; This file is part of GNU CLISP.
 ;;
@@ -128,11 +128,16 @@
   (setf (aref *callback-funcs* data) nil)
   nil)
 
-;;; this must come AFTER all callbacks because they use value1
-;;; undefine some clisp definitions used by gtk.h
+;; this must come AFTER all callbacks because they use value1
+;; undefine some clisp definitions used by gtk.h
 (c-lines "#undef unused~%")
 (c-lines "~{#undef value~d~%~}" '(1 2 3 4 5))
 
+;; w32api/basetyps.h defines interface to struct
+;; which breaks glade declarations like
+;; void glade_interface_destroy (GladeInterface *interface);
+(c-lines "#undef interface~%")
+
 (c-lines "#include <gtk/gtk.h>~%")
 
 ;;;
@@ -503,7 +508,8 @@ signal specific parameters, but not a data parameter."
   (gtk_main))
 
 ;;;
-;;; clisp gui
+;;; sample clisp gui
+;;; clisp -K full -x '(gtk:gui "ui.glade")'
 ;;;
 
 (defstruct gui main repl apropos status about-window about-text)
@@ -553,14 +559,16 @@ signal specific parameters, but not a data parameter."
   (gui-status-show (SYS::TEXT "Call DESCRIBE on the current selection")))
 
 (defun gui-quit (&optional (*gui* *gui*))
-  (gui-status-show (SYS::TEXT "Bye!"))
+  (gui-status-show (SYS::TEXT "Bye."))
   (gtk_main_quit)
   (throw 'gui-quit 0))
 
 (defun gui (file)
   (gtk_init nil nil)
   (let ((*gui* (gui-from-file file)))
-    (gui-status-show (SYS::TEXT "Welcome to CLISP!"))
+    (gui-status-show
+     (ext:string-concat (SYS::TEXT "Welcome to") " "
+                        (lisp-implementation-type) "!"))
     (gtk_widget_show (gui-main *gui*))
     (gtk_widget_hide (gui-about-window *gui*))
     (catch 'gui-quit (gtk_main))
