@@ -1,11 +1,11 @@
 #
 # This is the "link.sh" for the Oracle CLISP module
 #
-# $Id: link.sh,v 1.2 2004/02/25 16:22:59 haible Exp $
+# $Id: link.sh,v 1.3 2008/05/23 14:17:54 sds Exp $
 
-make clisp-module \
-     CC="${CC}" CPPFLAGS="${CPPFLAGS}" CFLAGS="${CFLAGS}" \
-     INCLUDES="$absolute_linkkitdir"
+${MAKE-make} clisp-module \
+  CC="${CC}" CPPFLAGS="${CPPFLAGS}" CFLAGS="${CFLAGS}" \
+  INCLUDES="$absolute_linkkitdir"
 
 NEW_FILES="oracle.o oiface.o orafns.o"
 
