@@ -1,6 +1,6 @@
 ;;; ANSI-compatible definitions + some extensions
 ;;; Bruno Haible 21.7.1994 - 2006
-;;; Sam Steingold 1999-2004, 2007
+;;; Sam Steingold 1999-2004, 2007-2009
 
 ;; ============================================================================
 
@@ -8,6 +8,7 @@
 (export '(declaim destructuring-bind complement
           constantly with-standard-io-syntax with-hash-table-iterator
           read-sequence write-sequence))
+(export '(ext::trim-if) "EXT")
 (in-package "SYSTEM")
 
 ;; ----------------------------------------------------------------------------
@@ -169,7 +170,7 @@
                     (unless nextch
                       (error-of-type 'end-of-file
                         :stream stream
-                        (TEXT "~S: input stream ~S ends within read macro beginning to ~S")
+                        (TEXT "~S: input stream ~S ends within read macro beginning with ~S")
                         'read stream ch
                     ) )
                     (unless (characterp nextch)
@@ -206,7 +207,7 @@
   (let ((vector-index
           (do ((i 0 (1+ i)))
                (nil)
-            (when (eq (%record-ref #'dispatch-reader i) vector) (return i)))))
+            (when (eq (closure-const #'dispatch-reader i) vector) (return i)))))
     (%defio #'dispatch-reader vector-index)
   )
 )
@@ -242,7 +243,7 @@
       (T (return))))
   (stream-element-type stream))
 
-(defun read-sequence (sequence stream &rest rest &key (start 0) (end nil))
+(defun %read-sequence (sequence stream &rest rest &key (start 0) (end nil))
   (declare (ignore start end))
   (let ((seltype (stream-input-element-type stream))
         (veltype (if (vectorp sequence) (array-element-type sequence) t)))
@@ -256,7 +257,13 @@
                   'read-sequence sequence stream
                   'read-char-sequence 'read-byte-sequence)))))
 
-(defun write-sequence (sequence stream &rest rest &key (start 0) (end nil))
+(defun read-sequence (sequence stream &rest rest &key (start 0) (end nil))
+  (declare (ignore start end))
+  (if (built-in-stream-p stream)
+      (apply #'%read-sequence sequence stream rest)
+      (apply 'gray::stream-read-sequence sequence stream rest)))
+
+(defun %write-sequence (sequence stream &rest rest &key (start 0) (end nil))
   (declare (ignore start end))
   (let ((seltype (stream-output-element-type stream))
         (veltype (if (vectorp sequence) (array-element-type sequence) t)))
@@ -273,6 +280,20 @@
                   'write-sequence sequence stream
                   'write-char-sequence 'write-byte-sequence)))))
 
+(defun write-sequence (sequence stream &rest rest &key (start 0) (end nil))
+  (declare (ignore start end))
+  (if (built-in-stream-p stream)
+      (apply #'%write-sequence sequence stream rest)
+      (apply 'gray::stream-write-sequence sequence stream rest)))
+
+(defun trim-if (predicate sequence)
+  (let ((beg (position-if-not predicate sequence)))
+    (if beg
+        (let ((end (1+ (position-if-not predicate sequence :from-end t))))
+          (if (and (= beg 0) (= end (length sequence))) sequence
+              (subseq sequence beg end)))
+        (subseq sequence 0 0))))  ; empty sequence of the same type as argument
+
 ;; ----------------------------------------------------------------------------
 
 ;; ANSI-CL specifies TYPE-ERRORs in many places.
@@ -405,13 +426,7 @@
 ;; ----------------------------------------------------------------------------
 
 (defmacro define-hash-table-test (name test hash)
-  (loop (when (symbolp name) (return))
-    ;; check-value will be defined in condition.lisp
-    (setq name (check-value
-                nil (make-condition 'simple-type-error
-                      :format-control (TEXT "~S: ~S should be a symbol")
-                      :format-arguments (list 'define-hash-table-test name)
-                      :datum name :expected-type 'symbol))))
+  (setq name (check-symbol name 'define-hash-table-test))
   `(progn (setf (get ',name 'hash-table-test) (cons #',test #',hash)) ',name))
 
 ;; (default-directory) is a Synonym for (cd).
