@@ -7,19 +7,27 @@
   (fresh-line) (prin1 object) (terpri) object)
 #+clisp (progn
 (defun kill-down (name)
-  (dolist (f (directory (ext:string-concat name "**")))
-    (format t "~&removing ~S~%" f)
-    (if (pathname-name f)
-        (delete-file f)
-        (ext:delete-directory f))))
+  (dolist (f (directory (ext:string-concat name "**/*")))
+    (format t "~&removing file ~S~%" f)
+    (delete-file f)))
 (defun rmrf (name)
-  (ext:dir (ext:string-concat name "**"))
+  (ext:dir (ext:string-concat name "**/*"))
+  (ext:dir (ext:string-concat name "**/"))
   (kill-down name)
-  (format t "~&removing ~S~%" name)
-  (ext:delete-directory name))
+  (dolist (d (directory (ext:string-concat name "**/")))
+    (format t "~&removing directory ~S~%" d)
+    (ext:delete-directory d)))
 (defun prepare-directory (name)
   (ensure-directories-exist name :verbose t)
   (kill-down name))
+(defun cmd-args ()
+  "command and arg list for cloning this clisp process"
+  (let* ((argv (ext:argv))
+         (lispinit (aref argv (1+ (position "-M" argv :test #'string=)))))
+    (values (aref argv 0)
+            (list "-q" "-norc" "-B" (namestring *lib-directory*) "-M"
+                  (namestring (merge-pathnames lispinit *lib-directory*))))))
+(export '(kill-down rmrf prepare-directory cmd-args))
 )
 (defun show-file (file)         ; return line count
   (with-open-file (st file :direction :input)
@@ -31,6 +39,13 @@
   (delete-file file)
   (delete-file (compile-file-pathname file))
   #+clisp (delete-file (make-pathname :type "lib" :defaults file)))
+(defun symbol-cleanup (s)
+  (setf (find-class s) nil) (makunbound s) (fmakunbound s) (unintern s))
+(defvar *run-test-truename*)
+;; export symbols used by tests
+(export '(princ-error show show-file finish-file post-compile-file-cleanup
+          type-error-handler
+          symbol-cleanup *run-test-truename* with-ignored-errors diff-seq))
 ;;; end helpers
 
 #+OLD-CLISP
@@ -252,13 +267,15 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
 (defvar *run-test-tester* #'do-test)
 (defvar *run-test-type* "tst")
 (defvar *run-test-erg* "erg")
-(defvar *run-test-truename*)
+(defvar *run-test-own-package* t)
 (defun run-test (testname
                  &key ((:tester *run-test-tester*) *run-test-tester*)
                       ((:ignore-errors *test-ignore-errors*)
                         *test-ignore-errors*)
                       ((:eval-method *eval-method*) *eval-method*)
                       (logname testname)
+                      ((:own-package *run-test-own-package*)
+                       *run-test-own-package*)
                  &aux (logfile (merge-extension *run-test-erg* logname))
                       error-count total-count *run-test-truename*)
   (with-open-file (s (merge-extension *run-test-type* testname)
@@ -270,20 +287,28 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
                                  #+(or CMU SBCL) :supersede
                                  #+ANSI-CL :if-exists #+ANSI-CL :new-version)
       (setq logfile (truename log))
-      (let* ((*package* *package*) (*print-circle* t) (*print-pretty* nil)
+      (let* ((*package*
+              (if *run-test-own-package*
+                  (make-package (namestring logfile)
+                                :use (cons *package*
+                                           (package-use-list *package*)))
+                  *package*))
+             (*print-circle* t) (*print-pretty* nil)
              (*eval-err* (make-string-output-stream))
              (*error-output* (make-broadcast-stream *error-output* *eval-err*))
              (*eval-out* (make-string-output-stream))
              (*standard-output* (make-broadcast-stream *standard-output*
                                                        *eval-out*)))
         (setf (values total-count error-count)
-              (funcall *run-test-tester* s log)))))
+              (funcall *run-test-tester* s log))
+        (when *run-test-own-package*
+          (delete-package *package*)))))
   (format t "~&~s: finished ~s (~:d error~:p out of ~:d test~:p)~%"
           'run-test testname error-count total-count)
   (if (zerop error-count)
       (delete-file logfile)
       (format t "~s: see ~a~%" 'run-test logfile))
-  (list testname total-count error-count))
+  (list logname total-count error-count))
 
 (defun report-results (res)
   "res = list of RUN-TEST return values (testname total-count error-count)"
@@ -307,77 +332,108 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
     (if files (report-results (mapcar #'run-test files))
         (warn "no ~S files in directories ~S" *run-test-type* dirlist))))
 
+(defparameter *all-tests*
+  '(#-(or AKCL ECL)   ("alltest")
+                      ("array")
+    #-OpenMCL         ("backquot")
+    #+CLISP           ("bin-io")
+    #-(and AKCL (not GCL)) ("characters")
+    #+(or CLISP ALLEGRO CMU OpenMCL LISPWORKS) ("clos")
+    #+CLISP           ("defhash")
+    #+(and CLISP UNICODE) ("encoding")
+                      ("eval20")
+    #+CLISP           ("ext-clisp")
+    #+(and CLISP FFI) ("ffi")
+                      ("floeps")
+    #-OpenMCL         ("format")
+    #+CLISP           ("genstream")
+    #+XCL             ("hash")
+                      ("hashlong")
+    #+CLISP           ("hashtable")
+                      ("iofkts")
+                      ("lambda")
+                      ("lists151")
+    #-(or GCL OpenMCL) ("lists152")
+                      ("lists153")
+                      ("lists154")
+                      ("lists155")
+                      ("lists156")
+                      ("list-set")
+    #+(or CLISP GCL ALLEGRO CMU SBCL OpenMCL LISPWORKS) ("loop")
+                      ("macro8")
+                      ("map")
+    #+(or CLISP ALLEGRO OpenMCL LISPWORKS) ("mop")
+                      ("number")
+    #+CLISP           ("number2")
+    #-(or AKCL ALLEGRO CMU OpenMCL) ("pack11")
+    #+(or XCL CLISP)  ("path")
+    #+XCL             ("readtable")
+    #-CMU             ("setf")
+    #+(and CLISP SOCKETS) ("socket")
+                      ("steele7")
+    #-ALLEGRO         ("streams")
+                      ("streamslong")
+                      ("strings")
+    #-(or AKCL ECL)   ("symbol10")
+                      ("symbols")
+                      ("time")
+    #+XCL             ("tprint")
+    #+XCL             ("tread")
+                      ("type")
+    #-(or)            ("unportable")
+    #+(and CLISP mt)  ("mt")
+    #+CLISP           ("weak")
+    #+(or CLISP ALLEGRO CMU19 OpenMCL LISPWORKS) ("weakhash")
+    #+(or CLISP LISPWORKS) ("weakhash2")
+                      ("bind" :eval-method :eval :logname "bind-eval")
+                      ("bind" :eval-method :compile :logname "bind-compile")
+    #+(or CLISP ALLEGRO CMU LISPWORKS) ("conditions" :ignore-errors nil)
+    #+CLISP           ("restarts" :ignore-errors nil)
+                      ("excepsit" :tester do-errcheck)
+    ))
+
+(defun test-weakptr ()
+  (let ((tmp (list "weakptr" 0 0)))
+    (dotimes (i 20 tmp)
+      (let ((weak-res (run-test "weakptr")))
+        (incf (second tmp) (second weak-res))
+        (incf (third tmp) (third weak-res))))))
+
 (defun run-all-tests (&key (disable-risky t)
                       ((:eval-method *eval-method*) *eval-method*))
   (let ((res ())
         #+CLISP (custom:*load-paths* nil)
         (*features* (if disable-risky *features*
                         (cons :enable-risky-tests *features*))))
-    (dolist (ff '(#-(or AKCL ECL)   "alltest"
-                                    "array"
-                  #-OpenMCL         "backquot"
-                  #+CLISP           "bin-io"
-                  #-(and AKCL (not GCL)) "characters"
-                  #+(or CLISP ALLEGRO CMU OpenMCL LISPWORKS) "clos"
-                  #+CLISP           "defhash"
-                  #+(and CLISP UNICODE) "encoding"
-                                    "eval20"
-                  #+CLISP           "ext-clisp"
-                  #+(and CLISP FFI) "ffi"
-                                    "floeps"
-                  #-OpenMCL         "format"
-                  #+CLISP           "genstream"
-                  #+XCL             "hash"
-                                    "hashlong"
-                  #+CLISP           "hashtable"
-                                    "iofkts"
-                                    "lambda"
-                                    "lists151"
-                  #-(or GCL OpenMCL) "lists152"
-                                    "lists153"
-                                    "lists154"
-                                    "lists155"
-                                    "lists156"
-                                    "list-set"
-                  #+(or CLISP GCL ALLEGRO CMU SBCL OpenMCL LISPWORKS) "loop"
-                                    "macro8"
-                                    "map"
-                  #+(or CLISP ALLEGRO OpenMCL LISPWORKS) "mop"
-                                    "number"
-                  #+CLISP           "number2"
-                  #-(or AKCL ALLEGRO CMU OpenMCL) "pack11"
-                  #+(or XCL CLISP)  "path"
-                  #+XCL             "readtable"
-                  #-CMU             "setf"
-                  #+(and CLISP SOCKETS) "socket"
-                                    "steele7"
-                  #-ALLEGRO         "streams"
-                                    "streamslong"
-                                    "strings"
-                  #-(or AKCL ECL)   "symbol10"
-                                    "symbols"
-                                    "time"
-                  #+XCL             "tprint"
-                  #+XCL             "tread"
-                                    "type"
-                  #-(or)            "unportable"
-                  #+(and CLISP mt)  "mt"
-                  #+CLISP           "weak"
-                  #+(or CLISP ALLEGRO CMU19 OpenMCL LISPWORKS) "weakhash"
-                  #+(or CLISP LISPWORKS) "weakhash2"))
-      (push (run-test ff) res))
-    (push (run-test "bind" :eval-method :eval :logname "bind-eval") res)
-    (push (run-test "bind" :eval-method :compile :logname "bind-compile") res)
+    (dolist (args *all-tests*)
+      (push (apply #'run-test args) res))
     #+(or CLISP ALLEGRO CMU SBCL LISPWORKS)
-    (let ((tmp (list "weakptr" 0 0)))
-      (push tmp res)
-      (dotimes (i 20)
-        (let ((weak-res (run-test "weakptr")))
-          (incf (second tmp) (second weak-res))
-          (incf (third tmp) (third weak-res)))))
-    #+(or CLISP ALLEGRO CMU LISPWORKS)
-    (push (run-test "conditions" :ignore-errors nil) res)
-    #+CLISP
-    (push (run-test "restarts" :ignore-errors nil) res)
-    (push (run-test "excepsit" :tester #'do-errcheck) res)
+    (push (test-weakptr) res)
     (report-results (nreverse res))))
+
+#+(and clisp mt)
+(defun run-all-tests-parallel (&key (concurrency 5) (disable-risky t)
+                               ((:eval-method *eval-method*) *eval-method*))
+  (let ((res ()) (lock (mt:make-mutex :name "res"))
+        (total (1+ (length *all-tests*)))
+        (custom:*load-paths* nil)
+        (*features* (if disable-risky *features*
+                        (cons :enable-risky-tests *features*))))
+    (macrolet ((run (body)
+                 `(mt:make-thread
+                   (lambda ()
+                     (let ((ans ,body))
+                       (mt:with-mutex-lock (lock)
+                         (push ans res))))
+                   :cstack-size 16777216)))
+      (dolist (args *all-tests*)
+        (let ((args args))
+          ;; this is "poor man" concurrency control - not exact at all
+          (loop until
+            (>= concurrency
+                (count t (mapcar #'mt:thread-active-p (mt:list-threads))))
+             :do (sleep 1))
+          (run (apply #'run-test args))))
+      (run (test-weakptr))
+      (loop :until (= total (length res)) :do (sleep 1))
+      (report-results (nreverse res)))))
